.block-editor-block-list__block[data-type="core/spacer"]:before{
  content:"";
  display:block;
  height:100%;
  min-height:8px;
  min-width:8px;
  position:absolute;
  width:100%;
  z-index:1;
}

.block-library-spacer__resize-container.has-show-handle,.wp-block-spacer.is-hovered .block-library-spacer__resize-container,.wp-block-spacer.is-selected.custom-sizes-disabled{
  background:#0000001a;
}
.is-dark-theme .block-library-spacer__resize-container.has-show-handle,.is-dark-theme .wp-block-spacer.is-hovered .block-library-spacer__resize-container,.is-dark-theme .wp-block-spacer.is-selected.custom-sizes-disabled{
  background:#ffffff26;
}

.block-library-spacer__resize-container{
  clear:both;
}
.block-library-spacer__resize-container:not(.is-resizing){
  height:100% !important;
  width:100% !important;
}
.block-library-spacer__resize-container .components-resizable-box__handle:before{
  content:none;
}
.block-library-spacer__resize-container.resize-horizontal{
  height:100% !important;
  margin-bottom:0;
}