.wp-block-accordion-heading__toggle{
  align-items:center;
  background:none;
  border:none;
  color:inherit;
  cursor:pointer;
  display:flex;
  font-family:inherit;
  font-size:inherit;
  font-style:inherit;
  font-weight:inherit;
  letter-spacing:inherit;
  line-height:inherit;
  overflow:hidden;
  padding:var(--wp--preset--spacing--20, 1em) 0;
  text-align:inherit;
  text-decoration:inherit;
  text-transform:inherit;
  width:100%;
  word-spacing:inherit;
}
.wp-block-accordion-heading__toggle:not(:focus-visible){
  outline:none;
}
.wp-block-accordion-heading__toggle:hover .wp-block-accordion-heading__toggle-title{
  text-decoration:underline;
}

.wp-block-accordion-heading__toggle-title{
  flex:1;
}

.wp-block-accordion-heading__toggle-icon{
  align-items:center;
  display:flex;
  height:1.2em;
  justify-content:center;
  width:1.2em;
}