.wp-block-table{
  margin:0 0 1em;
}
.wp-block-table td,.wp-block-table th{
  word-break:normal;
}
.wp-block-table :where(figcaption){
  color:#555;
  font-size:13px;
  text-align:center;
}
.is-dark-theme .wp-block-table :where(figcaption){
  color:#ffffffa6;
}