<?php

// autoload_psr4.php @generated by Composer

$vendorDir = dirname(__DIR__);
$baseDir = dirname($vendorDir);

return array(
    'Symfony\\Polyfill\\Php80\\' => array($baseDir . '/DataLiberation/vendor-patched/symfony/polyfill-php80/src'),
    'Symfony\\Polyfill\\Intl\\Normalizer\\' => array($baseDir . '/DataLiberation/vendor-patched/symfony/polyfill-intl-normalizer/src'),
    'Symfony\\Polyfill\\Ctype\\' => array($baseDir . '/DataLiberation/vendor-patched/symfony/polyfill-ctype/src'),
    'Rowbot\\URL\\' => array($baseDir . '/DataLiberation/vendor-patched/rowbot/url/src'),
    'Rowbot\\Punycode\\' => array($baseDir . '/DataLiberation/vendor-patched/rowbot/punycode/src'),
    'Rowbot\\Idna\\Resource\\' => array($baseDir . '/DataLiberation/vendor-patched/rowbot/idna/resources'),
    'Rowbot\\Idna\\' => array($baseDir . '/DataLiberation/vendor-patched/rowbot/idna/src'),
    'Psr\\Log\\' => array($baseDir . '/DataLiberation/vendor-patched/psr/log/src'),
    'Psr\\EventDispatcher\\' => array($baseDir . '/DataLiberation/vendor-patched/psr/event-dispatcher/src'),
    'Brick\\Math\\' => array($baseDir . '/DataLiberation/vendor-patched/brick/math/src'),
);
