.wp-block-cover.is-placeholder{
  align-items:stretch;
  display:flex;
  min-height:240px;
  padding:0 !important;
}
.wp-block-cover.is-placeholder .components-placeholder.is-large{
  justify-content:flex-start;
  z-index:1;
}
.wp-block-cover.is-placeholder:focus:after{
  min-height:auto;
}
.wp-block-cover.components-placeholder h2{
  color:inherit;
}
.wp-block-cover.is-transient{
  position:relative;
}
.wp-block-cover.is-transient:before{
  background-color:#fff;
  content:"";
  height:100%;
  opacity:.3;
  position:absolute;
  width:100%;
  z-index:1;
}
.wp-block-cover.is-transient .wp-block-cover__inner-container{
  z-index:2;
}
.wp-block-cover .components-spinner{
  left:50%;
  margin:0;
  position:absolute;
  top:50%;
  transform:translate(-50%, -50%);
}
.wp-block-cover .wp-block-cover__inner-container{
  margin-left:0;
  margin-right:0;
  text-align:left;
}
.wp-block-cover .wp-block-cover__placeholder-background-options{
  width:100%;
}
.wp-block-cover .wp-block-cover__image--placeholder-image{
  bottom:0;
  left:0;
  position:absolute;
  right:0;
  top:0;
}

[data-align=left]>.wp-block-cover,[data-align=right]>.wp-block-cover{
  max-width:420px;
  width:100%;
}

.block-library-cover__reset-button{
  margin-left:auto;
}

.block-library-cover__resize-container{
  bottom:0;
  left:0;
  min-height:50px;
  position:absolute !important;
  right:0;
  top:0;
}

.components-popover.block-editor-block-popover.block-library-cover__resizable-box-popover .block-library-cover__resize-container,.components-popover.block-editor-block-popover.block-library-cover__resizable-box-popover .components-popover__content>div{
  overflow:visible;
  pointer-events:none;
}

.block-editor-block-patterns-list__list-item .has-parallax.wp-block-cover{
  background-attachment:scroll;
}

.color-block-support-panel__inner-wrapper>:not(.block-editor-tools-panel-color-gradient-settings__item){
  margin-top:24px;
}