.wp-block-button.is-style-outline
	> .wp-block-button__link:not(.has-text-color, .has-background):hover {
	background-color: var(--wp--preset--color--contrast-2, var(--wp--preset--color--contrast, transparent));
	color: var(--wp--preset--color--base);
	border-color: var(--wp--preset--color--contrast-2, var(--wp--preset--color--contrast, currentColor));
}
